#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from logging import WARNING

# import interfaces
from pyams_form.interfaces.form import IWidgetsSuffixViewletsManager
from pyams_skin.layer import IPyAMSLayer
from pyams_security.interfaces import AuthenticatedPrincipalEvent, LOGIN_REFERER_KEY, ISecurityManager, ILoginView, \
    ISocialLoginConfiguration

# import packages
from authomatic.adapters import WebObAdapter
from authomatic.core import Authomatic
from pyams_template.template import template_config
from pyams_utils.registry import query_utility
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyramid.httpexceptions import HTTPNotFound
from pyramid.response import Response
from pyramid.security import remember
from pyramid.view import view_config


#
# Authomatic social login
#

@viewlet_config(name='social-login-suffix', layer=IPyAMSLayer,
                manager=IWidgetsSuffixViewletsManager, view=ILoginView, weight=50)
@template_config(template='templates/social-login.pt')
class SocialLoginViewletsSuffix(Viewlet):
    """Social login viewlets suffix"""

    def __new__(cls, *args, **kwargs):
        manager = query_utility(ISecurityManager)
        if not manager.enable_social_login:
            return None
        return Viewlet.__new__(cls)

    @property
    def use_popup(self):
        manager = query_utility(ISecurityManager)
        return manager.social_login_use_popup

    @property
    def providers(self):
        manager = query_utility(ISecurityManager)
        configuration = ISocialLoginConfiguration(manager)
        return sorted(configuration.values(),
                      key=lambda x: x.provider_id)


@view_config(route_name='login')
def login(request):
    """Login view for Authautomatic authentication"""
    # check security manager utility
    manager = query_utility(ISecurityManager)
    if (manager is None) or not manager.enable_social_login:
        raise HTTPNotFound()
    # store referrer
    session = request.session
    if LOGIN_REFERER_KEY not in session:
        session[LOGIN_REFERER_KEY] = request.referer
    # init authomatic
    provider_name = request.matchdict.get('provider_name')
    configuration = ISocialLoginConfiguration(manager).get_oauth_configuration()
    authomatic = Authomatic(config=configuration, secret=manager.authomatic_secret, logging_level=WARNING)
    # perform login
    response = Response()
    result = authomatic.login(WebObAdapter(request, response), provider_name)
    if result:
        if result.error:
            pass
        elif result.user:
            if not (result.user.id and result.user.name):
                result.user.update()
            social_folder = manager.get(manager.social_users_folder)
            user_id = '{provider_name}.{user_id}'.format(provider_name=provider_name,
                                                         user_id=result.user.id)
            request.registry.notify(AuthenticatedPrincipalEvent(plugin='oauth',
                                                                principal_id=user_id,
                                                                provider_name=provider_name,
                                                                user=result.user))
            principal_id = '{prefix}:{user_id}'.format(prefix=social_folder.prefix,
                                                       user_id=user_id)
            headers = remember(request, principal_id)
            response.headerlist.extend(headers)
        if manager.social_login_use_popup:
            response.text = result.popup_html()
        response.status_code = 302
        if LOGIN_REFERER_KEY in session:
            response.location = session[LOGIN_REFERER_KEY]
            del session[LOGIN_REFERER_KEY]
        else:
            response.location = '/'
    return response
